function [curve, auc, EER, ap]= roc(a)

%
% [curve, auc, EER, ap]= roc(a)
% Function computes Receiver Operator Characteristics (ROC) and derives the Area under Curve (AUC), Equal Error Rate (EER) and
% the interpolated Average Precision (AP) from the confidence submissions to the VCDT 2011. 
%
% References:
% this script is adapted from the EER/AUC script from the VCDT 2008
% http://www.imageclef.org/system/files/evaltool.tgz
%
%
% Input:
%       a:		struct contaning the prediction confidences and GT scores
%               for a particular class over all images of the test set
%
% Output:
%       curve:	ROC curve
%       auc:	AUC score for the class
%       EER:    EER score for the class
%       ap:     AP score for the class
%

gt=a(:,2);
confidence=a(:,1);

% compute true and false positive rates
[so,si]=sort(-confidence);
tp=cumsum(gt(si)>0)/sum(gt>0);
fp=cumsum(gt(si)==0)/sum(gt==0);
MFPR=1-fp;
TPR=tp;

EER=-1;


for i=1:size(MFPR)
%    printf("TPR: %f MFPR: %f dff: %f\n",TPR(i),MFPR(i),TPR(i)-MFPR(i));
    if TPR(i)>MFPR(i)
        EER=fp(i);
        break;
    end
end

[uo,ui]=unique(so);
tp=[0;tp(ui);1];
fp=[0;fp(ui);1];



% compute lower envelope and area under curve
di=[true ; tp(2:end-1)~=tp(1:end-2) ; true];
x=fp(di);
y=tp(di);
auc=(x(2:end)-x(1:end-1))'*y(1:end-1);


xp=[0 ; reshape([x x]',[],1) ; 1 ; 1];
yp=[0 ; 0 ; reshape([y y]',[],1) ; 1];
curve=[xp,yp];





% compute precision-recall curve 
% version with 'cumsum' is much faster
% 
% confidence_sorted = confidence(si);
% gt_sorted = gt(si);
% 
% prec = zeros(size(confidence));
% rec = zeros(size(confidence));
% 
% for i = 1:length(confidence_sorted)
%     % go through all examples starting from the hightest confidence
%     % on every stage calculate Precision and Recall for the top 'i' of found docs
%     % NB! here (I believe that) all top 'i' retrieved documents are
%     % considered to be positiv
%     
%     tp = sum(gt_sorted(1:i));
%     retrieved = i; % = (tp + fp)
%     relevant = sum(gt_sorted);
%     
%     prec(i) = tp/retrieved;
%     rec(i) = tp/relevant;    
%     
% end
% 
% % fp=cumsum(fp);
% % tp=cumsum(tp);
% % rec=tp/npos;
% % prec=tp./(fp+tp);
% 
% 
rec  = cumsum(gt(si)>0)/sum(gt>0);
%prec = cumsum(gt(si)>0)./(cumsum(gt(si)>0) + cumsum(gt(si)==0)); 
prec = cumsum(gt(si)>0)./ (1:length(gt))';



% compute interpolated Average Precision

ap=0;
for t=0:0.1:1
    p=max(prec(rec>=t));
    if isempty(p)
        p=0;
    end
    ap=ap+p/11;
end


end %function
